#ifndef EVENT_H_
#define EVENT_H_

/************************************************************
This header is blatenlty stolen from SDL_event.h because it
works and I like it.  I simplified it because I'm lazy.
************************************************************/

const int RELEASED=0;
const int PRESSED=1;

// Event enumerations
enum {  NOEVENT = 0,			// Unused (do not remove)
        ACTIVEEVENT,			// Application loses/gains visibility
        KEYDOWN,			   // Keys pressed
        KEYUP,			      // Keys released
        MOUSEMOTION,			// Mouse moved
        MOUSEBUTTONDOWN,	// Mouse button pressed
        MOUSEBUTTONUP,		// Mouse button released
        MOUSEWHEEL,        // Mouse wheel event
        JOYAXISMOTION,		// Joystick axis motion
        JOYBALLMOTION,		// Joystick trackball motion
        JOYHATMOTION,		// Joystick hat position change
        JOYBUTTONDOWN,		// Joystick button pressed
        JOYBUTTONUP,			// Joystick button released
        QUIT,			      // User-requested quit
        VIDEORESIZE,			// User resized video mode
        VIDEOEXPOSE,			// Screen needs to be redrawn
       // Events  USEREVENT through  MAXEVENTS-1 are for your use
        USEREVENT = 24,
       // This last event is only for bounding internal arrays
       //It is the number of bits in the event mask datatype -- Uint32

        NUMEVENTS = 32
};

// Application visibility event structure
typedef struct NIGHT_ActiveEvent
{
	int type;	   // ACTIVEEVENT
	int gain;	   // Whether given states were gained or lost (1/0)
	int state;	   // A mask of the focus states
} NIGHT_ActiveEvent;

// Keyboard event structure
typedef struct NIGHT_KeyboardEvent
{
	int type;	// KEYDOWN or KEYUP
	int state;	// PRESSED or RELEASED
   int key;    // key
} NIGHT_KeyboardEvent;

// Mouse motion event structure
typedef struct NIGHT_MouseMotionEvent
{
	int type;	// MOUSEMOTION
	int x, y;	// The X/Y coordinates of the mouse
	int xrel;	// The relative motion in the X direction
	int yrel;	// The relative motion in the Y direction
} NIGHT_MouseMotionEvent;

// Mouse button event structure
typedef struct NIGHT_MouseButtonEvent
{
	int type;	// MOUSEBUTTONDOWN or MOUSEBUTTONUP
   int state;	// NIGHT_PRESSED or NIGHT_RELEASED
	int button;	// The mouse button index
	int x, y;	// The X/Y coordinates of the mouse at press time
} NIGHT_MouseButtonEvent;

// Mouse wheel event structure
typedef struct NIGHT_MouseWheelEvent
{
	int type;	// MOUSEWHEEL
   int pos;	   // wheel position
	int relpos;	// reletive position
} NIGHT_MouseWheelEvent;

// Joystick axis motion event structure
typedef struct NIGHT_JoyAxisEvent
{
	int type;	   // JOYAXISMOTION
	int which;	// The joystick device index
	int axis;	   // The joystick axis index
	int value;	// The axis value (range: -32768 to 32767)
} NIGHT_JoyAxisEvent;

// Joystick trackball motion event structure
typedef struct NIGHT_JoyBallEvent
{
	int type;	   // JOYBALLMOTION
	int which;	// The joystick device index
	int ball;	   // The joystick trackball index
	int xrel;	// The relative motion in the X direction
	int yrel;	// The relative motion in the Y direction
} NIGHT_JoyBallEvent;

// Joystick hat position change event structure
typedef struct NIGHT_JoyHatEvent
{
	int type;	   // JOYHATMOTION
	int which;	// The joystick device index
	int hat;	   // The joystick hat index
	int value;	// The hat position value:
			    //HAT_LEFTUP   HAT_UP       HAT_RIGHTUP
			    //HAT_LEFT     HAT_CENTERED HAT_RIGHT
			    //HAT_LEFTDOWN HAT_DOWN     HAT_RIGHTDOWN
			    //Note that zero means the POV is centered.

} NIGHT_JoyHatEvent;

// Joystick button event structure
typedef struct NIGHT_JoyButtonEvent
{
	int type;	   // JOYBUTTONDOWN or JOYBUTTONUP
	int which;	// The joystick device index
	int button;	// The joystick button index
	int state;	// PRESSED or RELEASED
} NIGHT_JoyButtonEvent;

// The "window resized" event
typedef struct NIGHT_ResizeEvent
{
	int type;	// VIDEORESIZE
	int w;		// New width
	int h;		// New height
} NIGHT_ResizeEvent;

// The "screen redraw" event
typedef struct NIGHT_ExposeEvent
{
	int type;	   // VIDEOEXPOSE
} NIGHT_ExposeEvent;

// The "quit requested" event
typedef struct NIGHT_QuitEvent
{
	int type;	   // QUIT
} NIGHT_QuitEvent;

// A user-defined event type
typedef struct NIGHT_UserEvent
{
	int type;	   // USEREVENT through NUMEVENTS-1
	int code;	   // User defined event code
	void *data1;	// User defined data pointer
	void *data2;	// User defined data pointer
} NIGHT_UserEvent;



// General event structure
typedef union
{
	int type;
	NIGHT_ActiveEvent active;
	NIGHT_KeyboardEvent key;
	NIGHT_MouseMotionEvent mmotion;
	NIGHT_MouseButtonEvent mbutton;
	NIGHT_MouseWheelEvent mwheel;
	NIGHT_JoyAxisEvent jaxis;
	NIGHT_JoyBallEvent jball;
	NIGHT_JoyHatEvent jhat;
	NIGHT_JoyButtonEvent jbutton;
	NIGHT_ResizeEvent resize;
	NIGHT_ExposeEvent expose;
	NIGHT_QuitEvent quit;
	NIGHT_UserEvent user;
} NIGHT_Event;


#endif

