#include "eventqueue.h"

EventQueue::EventQueue()
{
   for(int key=0; key<GLFW_KEY_LAST+1; key++)
      {
         myKeys[key]=GLFW_RELEASE;
         myLastKeys[key]=GLFW_RELEASE;
      }

   for(int button=0; button<GLFW_MOUSE_BUTTON_LAST+1; button++)
   {
      myMouseButtons[button]=GLFW_RELEASE;
      myLastMouseButtons[button]=GLFW_RELEASE;
   }
}
EventQueue::~EventQueue()
{
   NIGHT_Event* tempEvent;
   while(myQueue.empty()!=true)
   {
      tempEvent=myQueue.front();
      delete tempEvent;
      myQueue.pop();
   }
}

NIGHT_Event* EventQueue::event()
{
   static NIGHT_Event event;
   NIGHT_Event* temp;

   if(myQueue.empty()==true)
   {
      return (NIGHT_Event*)0;
   }

   temp=myQueue.front();
   event=(*temp);
   delete temp;
   myQueue.pop();

   return &event;
}

void EventQueue::processEvents()
{
   //clear the queue if an event wasn't handled -it should have been
   NIGHT_Event* tempEvent;
   while(myQueue.empty()==false)
   {
      tempEvent=myQueue.front();
      delete tempEvent;
      myQueue.pop();
   }

   checkKeyboardEvents();
   checkMouseEvents();
   checkJoystickEvents();
   checkSystemEvents();
}

void EventQueue::checkKeyboardEvents()
{
  //compare keys
   //if they're not the same, something changed and we must create an event
   for(int key=0; key<GLFW_KEY_LAST+1; key++)
   {
      myKeys[key]=glfwGetKey(key);
      if(myKeys[key]==GLFW_PRESS &&
         myLastKeys[key]==GLFW_RELEASE)
      {
         //create a keypress event
         NIGHT_Event* keyEvent=new NIGHT_Event;
         keyEvent->type=KEYDOWN;
         keyEvent->key.key=key;
         keyEvent->key.state=PRESSED;
         myQueue.push(keyEvent);
      }

      if(myKeys[key]==GLFW_RELEASE &&
         myLastKeys[key]==GLFW_PRESS)
      {
         //create a keyrelease event
         NIGHT_Event* keyEvent=new NIGHT_Event;
         keyEvent->type=KEYUP;
         keyEvent->key.key=key;
         keyEvent->key.state=RELEASED;
         myQueue.push(keyEvent);
      }

      myLastKeys[key]=myKeys[key];
   }
}

void EventQueue::checkMouseEvents()
{
   glfwGetMousePos(&myMouseX, &myMouseY);

   for(int button=0; button<GLFW_MOUSE_BUTTON_LAST+1; button++)
   {
      myMouseButtons[button]=glfwGetMouseButton(button);

      if(myMouseButtons[button]==GLFW_PRESS &&
         myLastMouseButtons[button]==GLFW_RELEASE)
      {
         //mouse button press
         NIGHT_Event* mouseEvent=new NIGHT_Event;
         mouseEvent->type=MOUSEBUTTONDOWN;
         mouseEvent->mbutton.state=PRESSED;
         mouseEvent->mbutton.button=button;
         mouseEvent->mbutton.x=myMouseX;
         mouseEvent->mbutton.y=myMouseY;
         myQueue.push(mouseEvent);
      }

      if(myMouseButtons[button]==GLFW_RELEASE &&
         myLastMouseButtons[button]==GLFW_PRESS)
      {
         //mouse button release
         NIGHT_Event* mouseEvent=new NIGHT_Event;
         mouseEvent->type=MOUSEBUTTONUP;
         mouseEvent->mbutton.state=RELEASED;
         mouseEvent->mbutton.button=button;
         mouseEvent->mbutton.x=myMouseX;
         mouseEvent->mbutton.y=myMouseY;
         myQueue.push(mouseEvent);
      }

      myLastMouseButtons[button]=myMouseButtons[button];
   }

   if((myMouseX!=myLastMouseX) || (myMouseY!=myLastMouseY))
   {
      //mouse move
      NIGHT_Event* mouseEvent=new NIGHT_Event;
      mouseEvent->type=MOUSEMOTION;
      mouseEvent->mmotion.x=myMouseX;
      mouseEvent->mmotion.y=myMouseY;
      mouseEvent->mmotion.xrel=(myMouseX-myLastMouseX);
      mouseEvent->mmotion.yrel=(myMouseY-myLastMouseY);

      myLastMouseX=myMouseX;
      myLastMouseY=myMouseY;
      myQueue.push(mouseEvent);
   }

   myMouseWheel=glfwGetMouseWheel();
   if(myMouseWheel!=myLastMouseWheel)
   {
      //mouse wheel
      NIGHT_Event* mouseEvent=new NIGHT_Event();
      mouseEvent->type=MOUSEWHEEL;
      mouseEvent->mwheel.pos=myMouseWheel;
      mouseEvent->mwheel.relpos=(myMouseWheel-myLastMouseWheel);

      myLastMouseWheel=myMouseWheel;
      myQueue.push(mouseEvent);
   }
}

void EventQueue::checkJoystickEvents()
{

}

void EventQueue::checkSystemEvents()
{

}

