#include "window.h"
#include <iostream>


Window::Window(int width, int height, int redbpp, int greenbpp, int bluebpp,
               int alphabpp, int depth, int stencil, bool fullScreen,
               std::string title):
myWidth(width),
myHeight(height),
myRedBits(redbpp),
myGreenBits(greenbpp),
myBlueBits(bluebpp),
myAlphaBits(alphabpp),
myDepthBits(depth),
myStencilBits(stencil),
myIsFullScreen(fullScreen),
myTitle(title)
{
   myEventQueue=new EventQueue();
   if (glfwInit()==GL_FALSE)
   {
      //throw();
   }

   int screen;
   if(fullScreen==true)
   {
      screen=GLFW_FULLSCREEN;
   }
   else
   {
      screen=GLFW_WINDOW;

   }

   if(glfwOpenWindow( width, height, redbpp, greenbpp, bluebpp, alphabpp,
                      depth, stencil, screen)==GL_FALSE)
   {
      //throw();
   }

   if(screen=GLFW_WINDOW)
   {
      glfwSetWindowTitle(title.c_str());
   }
}

Window::~Window()
{
   //delete the event queue
   delete myEventQueue;

   //close out the window stuff
   glfwCloseWindow();
   glfwTerminate();

   //delete all the cursors
   CursorMapIter iter;
   for(iter=myCursors.begin(); iter!=myCursors.end(); ++iter)
   {
         delete iter->second;
   }
}

void Window::flip()
{
   if(myCursorVisible==true && myCurrentCursor!=NULL)
   {
      set2D();
      myCurrentCursor->draw();
   }
   glfwSwapBuffers();
   myEventQueue->processEvents();
}

void Window::setVertRetraceWait(int num)
{
      glfwSwapInterval(num);
}

void Window::set3D() const
{
   //spinny triangle so that I can see it doing something
   glMatrixMode(GL_PROJECTION);	//clear the projection matrix
   glLoadIdentity();
   //now reset the projection matrix for a perspective viewing
   gluPerspective(45.0f, (GLfloat)(myWidth)/(GLfloat)(myHeight),1.0f,100.0f);

   //clear the model matrix for drawing our favorite tri
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

}

void Window::set2D() const
{
      //clear the projection matrix
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      //reset the projection matrix for 2D drawing
      gluOrtho2D(0, (GLfloat)(myWidth),(GLfloat)(myHeight),0);
      //clear the model matrix so raster position is in reference to the origin
      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity();
}

void Window::setCursor(std::string name)
{
   CursorMapIter iter;
   iter=myCursors.find(name);
   if(iter==myCursors.end())
   {
      std::cout << "cannot find cursor: " << name << std::endl;
      if(myCursors.empty()==false)
      {
         myCurrentCursor=myCursors.begin()->second;
         return;
      }
      else
      {
         myCurrentCursor=NULL;
         return;
      }
   }
   myCurrentCursor=iter->second;
}

void Window::addCursor(std::string name, Cursor* newCursor)
{
   CursorMapIter iter;
   iter=myCursors.find(name);
   if(iter!=myCursors.end())
   {
      //replace cursor
      delete iter->second;
      iter->second=newCursor;
   }
   myCursors.insert(CursorMapPair(name, newCursor));
}

