#ifndef NIGHT_WINDOW_H
#define NIGHT_WINDOW_H

#include <string>
#include <map>

#include <GL/glfw.h>

#include "UI_cursor.h"
#include "event.h"
#include "eventqueue.h"
#include "font_repository.h"

class Window
 {
     public:
         Window(int width, int height, int redbpp, int greenbpp, int bluebpp,
               int alphabpp, int depth, int stencil, bool fullScreen,
               std::string title);

         virtual ~Window();

         virtual NIGHT_Event* event() {return myEventQueue->event();}
         virtual bool hasEvent() {return myEventQueue->hasEvent();}


         virtual void set3D() const;
         virtual void set2D() const;

         virtual void flip();
         virtual void setVertRetraceWait(int num);

         virtual int width() const {return myWidth;}
         virtual int height() const {return myHeight;}

         virtual void setCursor(std::string name);
         virtual void showCursor() {myCursorVisible=true;}
         virtual void hideCursor() {myCursorVisible=false;}
         virtual void addCursor(std::string name, Cursor* newCursor);

         virtual const FontRepository* fontRepository() const {return &myFontRepository;}

     protected:
         //window stuff
         int myWidth;
         int myHeight;
         int myRedBits;
         int myGreenBits;
         int myBlueBits;
         int myAlphaBits;
         int myDepthBits;
         int myStencilBits;
         bool myIsFullScreen;
         std::string myTitle;

         //event stuff
         EventQueue* myEventQueue;

         //cursor stuff
         CursorMap myCursors;
         Cursor* myCurrentCursor;
         bool  myCursorVisible;

         //Font Repository
         FontRepository myFontRepository;
 };


#endif

