#include "bitmap_font.h"

#include "GL/gl.h"

#include "helvetica10.h"
#include "helvetica12.h"
#include "helvetica8b.h"
#include "helvetica10b.h"
#include "helvetica12b.h"
#include "helvetica14b.h"
#include "screen12.h"
#include "screen14.h"
#include "screen15.h"


bitmapFont::bitmapFont(std::string fontName)
{
   myName=fontName;

   if(fontName=="HELVETICA10")
   {
      font_data=hel_10_font_data;
   }

   else if(fontName=="HELVETICA12")
   {
      font_data=hel_12_font_data;
   }

   else if(fontName=="HELVETICA_BOLD8")
   {
      font_data=hel_bld_8_font_data;
   }

   else if(fontName=="HELVETICA_BOLD10")
   {
      font_data=hel_bld_10_font_data;
   }

   else if(fontName=="HELVETICA_BOLD12")
   {
      font_data=hel_bld_12_font_data;
   }

   else if(fontName=="HELVETICA_BOLD14")
   {
      font_data=hel_bld_14_font_data;
   }

   else if(fontName=="SCREEN12")
   {
      font_data=screen_12_font_data;
   }

   else if(fontName=="SCREEN14")
   {
      font_data=screen_14_font_data;
   }

   else if(fontName=="SCREEN15")
   {
      font_data=screen_15_font_data;
   }
   else
   {
      font_data=hel_10_font_data;
   }
}

bitmapFont::~bitmapFont()
{
}

void bitmapFont::print(char* str,...) const
{
    va_list arg;
	char text[256];

	va_start(arg,str);
	  vsprintf(text,str,arg);
	va_end(arg);

    GLint alignment;
	unsigned char c;

	glGetIntegerv(GL_UNPACK_ALIGNMENT, &alignment);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

   int counter=0;
   while(text[counter]!=0)
   {
      c=(unsigned char)(text[counter]);
      counter++;

		const char_data_type& cd = font_data.chars[c];
		if (cd.data_offset==-1)
		{
			GLubyte nullBitmap = 0;
			glBitmap(1, 1, 0, 0, cd.advance, 0, &nullBitmap);
		}
		else
		{
			GLubyte *bitmap = &font_data.bitmap_data[cd.data_offset];
			glBitmap(cd.width, cd.height, cd.xorig, cd.yorig, cd.advance, 0, bitmap);
		}
	}
	glPixelStorei(GL_UNPACK_ALIGNMENT, alignment);
}

void bitmapFont::print(int x, int y, char* str, ...) const
{
    va_list arg;
	char text[256];

	va_start(arg,str);
	  vsprintf(text,str,arg);
	va_end(arg);

    glRasterPos2i(x, y);
    print(text);
}

void bitmapFont::print(float x, float y, char* str,...) const
{
    va_list arg;
	char text[256];

	va_start(arg,str);
	  vsprintf(text,str,arg);
	va_end(arg);

    glRasterPos2f(x, y);
    print(text);
}

void bitmapFont::print(float x, float y, float z, char* str, ...) const
{
   va_list arg;
	char text[256];

	va_start(arg,str);
	  vsprintf(text,str,arg);
	va_end(arg);

    unsigned char c;
	float pos = 0;

	int baseLine = -(font_data.ymin);


	glBegin(GL_QUADS);
   int counter=0;
   while(text[counter]!=0)
   {
      c=(unsigned char)(text[counter]);
      counter++;

		const char_data_type & cd = font_data.chars[c];
		if (cd.data_offset != -1)
		{
			float cellX = (c%16)/16.0;
			float cellY = (c/16)/16.0;

			glTexCoord2f(cellX + 1.0/16.0, cellY);
			glVertex3f(x + pos + 16.0, -baseLine + y + 0.0, z);

			glTexCoord2f(cellX + 1.0/16.0, cellY + 1.0/16.0);
			glVertex3f(x + pos + 16.0, -baseLine + y + 16.0, z);

			glTexCoord2f(cellX, cellY + 1.0/16.0);
			glVertex3f(x + pos + 0.0, -baseLine + y + 16.0, z);

			glTexCoord2f(cellX, cellY);
			glVertex3f(x + pos + 0.0, -baseLine + y + 0.0, z);
		}
		pos += cd.advance;
	}
	glEnd();
}


//some size information
int bitmapFont::string_width(char* str, ...) const
{
   va_list arg;
	char text[256];

	va_start(arg,str);
	  vsprintf(text,str,arg);
	va_end(arg);

   unsigned char c;
	int length = 0;

   int counter=0;
   while(text[counter]!=0)
   {
      c=(unsigned char)(text[counter]);
      counter++;

		length += font_data.chars[c].advance;
	}

	return length;
}

int bitmapFont::string_height(char* str, ...) const
{
   va_list arg;
	char text[256];

	va_start(arg,str);
	  vsprintf(text,str,arg);
	va_end(arg);

   unsigned char c;
	int height = 0;

   int counter=0;
   while(text[counter]!=0)
   {
      c=(unsigned char)(text[counter]);
      counter++;

		if(height<= font_data.chars[c].height)
		{
		   height=font_data.chars[c].height;
		}
	}

	return height;
}
