#ifndef UI_AREA
#define UI_AREA

#include <vector>

#include "UI_composite_element.h"

//forward defs
class UI_space;
class UI_block;

//UI area flags
const int SHOW_HEADER=21;
const int HEADER_TOP=22;
const int HEADER_BOTTOM=23;


/**********************************************************
Area Class:
The area is where a space is drawn.  An area can hold any of
the known types of spaces.  The space info is not shared between
the different areas.  Each area has it's own copy of the spaces.
For example, a 3D modeling space will have a different view
of the the data (which is shared) than another area which
would allow for simultaneous, different views of the 3D data.

An area may hold one of each type of spaces, but no more than
one of that kind. It can only show one at a time

Areas are split and joined through the edges that bound them
screen_vert ordering in a Areas:

2---------3
|         |
|         |
1---------4
**********************************************************/

typedef std::vector<UI_space*> spaceVector;
typedef spaceVector::iterator spaceVectorIter;

class UI_area : public UI_composite_element
{
	public:
		UI_area();
		virtual ~UI_area();

		UI_area(const UI_area &copy_area);  //copy constructor for splitting areas

		virtual void draw();
		virtual void process_event(NIGHT_Event event);

	protected:
		spaceVector mySpaces;  //might want to look at default having each space object
		spaceVectorIter myCurrentSpace;
};

class UI_area_creator: public UI_element_creator
{
	public:
		UI_area_creator() {guiFactory::register_creator("AREA", this);}
		~UI_area_creator() {}

		UI_area* create() {return new UI_area;}

	protected:
		UI_area* temp;
} area_instance;


#endif
