#include "UI_block.h"


void UI_block::draw()
{
	UIListIter list_iter;

	glPushName((GLuint)(this));
	glPushMatrix();
	glTranslatef(pos.x,pos.y, 0.1);
	glColor4f(col.r, col.g, col.b, col.a);

	draw_box(GL_POLYGON);

	for(list_iter=elements.begin(); list_iter!=elements.end(); list_iter++)
	{
		if(list_iter!=NULL)
		{
			(*list_iter)->draw();
		}
	}

	glPopMatrix();
	glPopName();
}

void UI_block::process_event(NIGHT_Event event)
{
	UIListIterss list_iter;

	if(checkFlag(MOUSE_OVER)==true)
	{
		for(list_iter=elements.begin(); list_iter!=elements.end(); list_iter++)
		{
			if(list_iter!=NULL)
			{
				(*list_iter)->process_event(event);
			}
		}
	}
}

void UI_block::arrange()
{
	int temp_x, temp_y;
	int size_x, size_y;

	std::list<UI_element*>::iterator list_iter;

	temp_x=0; temp_y=0;
	size_x=0; size_y=0;

	if(check_flag(VERTICLE)==true)
	{
		//move each one above the next
		for(list_iter=elements.begin(); list_iter!=elements.end(); list_iter++)
		{
			(*list_iter)->set_pos(0, temp_y);
			(*list_iter)->get_size(size_x, size_y);

			//step to the next verticle position
			temp_y+=size_y;
			//find the bigest size_x;
			if(size_x>temp_x)
			{
				//temp_x holds the largest size_x, which we'll need in a second
				temp_x=size_x;
			}
		}
		//go back and resize the x dimensions so they're all the same size
		for(list_iter=elements.begin(); list_iter!=elements.end(); list_iter++)
		{
			(*list_iter)->get_size(size_x, size_y);
			(*list_iter)->set_size(temp_x, size_y);
		}
		return;
	}
	else if(check_flag(HORIZONTAL)==true)
	{
		//move each one next the next
		for(list_iter=elements.begin(); list_iter!=elements.end(); list_iter++)
		{
			(*list_iter)->set_pos(temp_x, 0);
			(*list_iter)->get_size(size_x, size_y);

			//step to the next verticle position
			temp_x+=size_x;
			//find the bigest size_y;
			if(size_y>temp_y)
			{
				//temp_y holds the largest size_y, which we'll need in a second
				temp_y=size_y;
			}
		}
		//go back and resize the x dimensions so they're all the same size
		for(list_iter=elements.begin(); list_iter!=elements.end(); list_iter++)
		{
			(*list_iter)->get_size(size_x, size_y);
			(*list_iter)->set_size(size_x, temp_y);
		}

		return;
	}
	else if(check_flag(SQUARE)==true)
	{
		//not implemented yet because I'm dumb
		return;
	}
}
/************************************************************
UI_Block menu parser

Input: coded string to be parsed for the menu items
Outputs: none
Actions: Sets up a class UI_blocks menu values

Parse menu description strings, string is of the form:
"Title=ssss|{ssss}"
-Title=indicates the menu title
-ssss indicates an option
the return value is the index of the option (starting with 1)
A pipe (|) is the seperator
************************************************************/

void UI_block::parse_menu_string(char* input_string)
{

	char *token;
	UI_button *temp_button;
	UI_text_element *temp_text;
	std::list<UI_element*>::iterator list_iter;

	//clear out the current list
	for(list_iter=elements.begin(); list_iter!=elements.end(); list_iter++)
	{
		if(*list_iter!=NULL)
		{
			delete (*list_iter);
		}
	}

	//parse through the string
	token = strtok(input_string,"|");
	while (token != NULL)
	{
		token = strtok(NULL, "|=");
		if (token=="Title")
		{
			//get the next token for the title name
			token = strtok(NULL, "|=");
			temp_text=(UI_text*)UI_factory::create_element("TEXT_ELEMENT");
			temp_text->set_string(token);
			elements.push_back(temp_text);
		}
		else
		{
			temp_button=(UI_button*)UI_factory::create_element("BUTTON");
			temp_text=(UI_text*)UI_factory::create_element("TEXT_ELEMENT");
			temp_text->set_string(token);
			temp_button->add(temp_text);
			elements.push_back(temp_button);
		}
	}
}

UI_block* UI_block_creator::create(string menu_string)
{
	return new UI_block(menu_string);
}

UI_block* UI_block_creator::create(TiXmlNode* node)
{
	temp=new UI_block();
	xml_fill(node, temp);
	return temp;
}

void UI_block_creator::xml_fill(TiXmlNode* node, UI_block* temp_block)
{
	TiXmlNode* current_node;
	TiXmlElement* element;
	TiXmlElement* child_element;
	TiXmlText* text;

	const char* name;

	UI_element_creator::xml_fill(node, temp_area);

	current_node=node;
	element=current_node->FirstChildElement();

	while(element!=NULL)
	{
		name=element->Value();
		if (name== "Title")
		{
			UI_text_element* new_title;
			new_title=(UI_text_element*)UI_factory::create_text_element("TEXT_ELEMENT",element)
			temp_block->add(new_title);
		}
		else if (name=="Block")
		{
			UI_block* new_block;
			new_block=(UI_block*)UI_factory::create_element("BLOCK", element);
			temp_block->add(new_block);
		}
		else if (name=="Button")
		{
			UI_button* new_button;
			new_button=(UI_button*)UI_factory::create_element("BUTTON", element);
			temp_block->add(new_block);
		}
		else if (name=="Alignment")
		{
			child_element=element->FirstChildElement();
			text=child_element->ToText();
			if(text->Value()=="Vertical")
			{
				temp_block->set_flag(VERTICLE, true);
			}
			else if (text->Value()=="Horizontal")
			{
				temp_block->set_flag(HORIZONTAL, true);
			}
			else if (text->Value()=="Square")
			{
				temp_block->set_flag(SQUARE, true);
			}
		}
		element=current_node->NextSiblingElement();
	}
}




