#include "UI_edge.h"


/************************************************************
Edge Constructor

Input: None
Outputs: none
Actions: Sets up a default UI_edge
************************************************************/
UI_edge::UI_edge()
{
}

UI_edge::UI_edge(UI_area* new_area)
{
	my_area=new_area;
}

/************************************************************
Edge Destructor

Input: None
Outputs: none
Actions: Deletes an UI_edge
************************************************************/
UI_edge::~UI_edge()
{
	split_menu=new UI_block();

}

UI_edge::UI_edge(const UI_edge &new_edge)
{
	delete split_menu;
}

/************************************************************
Edge Draw function

Input: draw flags
Outputs: none
Actions: Draws an UI_edge with the draw flags
************************************************************/
void UI_edge::draw()
{
	glPushName((GLuint)(this));
	glPushMatrix();
	glTranslatef(pos.x,pos.y, 0.1);

	glColor3f(col.r, col.g, col.b);
	glBegin(GL_LINES);
		glVertex2f(pos.x, pos.y);
		glVertex2f(size.x, size.y);
	glEnd();

	if(check_flag(SHOW_SPLIT_MENU)==true)
	{
		split_menu->draw();
	}
	glPopMatrix();
	glPopName();
}

/************************************************************
Edge Process Events

Input: Event Type
Outputs: none
Actions: Process any events that this UI_edge may be concerened
about (such as the split/join/header menu)
************************************************************/
void UI_edge::process_event(NIGHT_Event event)
{
	screen_vert mouse;
	int decision=-1;

	if(check_flag(MOUSE_OVER)==true)
	{
		switch (event.type)
		{
			case MOUSEMOTION:
			{
				//change the color because it's active
				set_color(1.0,0.0,0.0);
				set_flag(ACTIVE,true);

				if(is_horizontal()==true)
				{
					NIGHT::set_cursor(NORTH_SOUTH);
				}
				else
				{
					NIGHT::set_cursor(EAST_WEST);
				}
				break;
			}
			case MOUSEBUTTONDOWN:
			{

				set_flag(SHOW_SPLIT_MENU,true);
				NIGHT::get_mouse_coords(&mouse);
				split_menu->set_pos(mouse);

				UI_area* temp_area;
				temp_area=dynamic_cast<UI_screen*>(parent)->get_active_area();
				if(temp_area->check_flag(SHOW_HEADER)==true)
				{
					split_menu->set_popup_menu("Split Area|Join Areas|No Header");
				}
				else
				{
					split_menu->set_popup_menu("Split Area|Join Areas|Add Header");
				}


				while(decision==-1)
				{
					dynamic_cast<UI_screen*>(parent)->draw();
					SDL_PollEvent(&event);
					split_menu->process_event(event);
					decision=split_menu->get_return();
				}

				switch (decision)
				{
					case 1:   //split areas
					{
						dynamic_cast<UI_screen*>(parent)->interactively_split();
						break;
					}
					case 2:   //join areas
					{
						dynamic_cast<UI_screen*>(parent)->join_areas();
						break;
					}
					case 3:   //add header/no header
					{
						temp_area->toggle_flag(SHOW_HEADER);
						break;
					}
					default:
					{
						break;
					}
				} //end decision switch
			} //end case mouse button press
			default:
			{
				break;
			}
		} //end event switch
	} //end if mouse over
	else
	{
		//it's inactive
		set_color(1.0,1.0,1.0);
		set_flag(ACTIVE,false);
	}

	set_flag(MOUSE_OVER, false);
}

bool UI_edge::is_border()
{
	if((area1==NULL) or (area2==NULL))
	{
		 return true;
	}
	else
	{
		 return false;
	}
}

