#ifndef UI_SCREEN
#define UI_SCREEN

#include "prerequisites.h"
#include "UI_element.h"

//forward defs
class UI_area;
class UI_edge;

//some defines for the UI_screen and areas (size in pixels)
const int AREAMINX=32;
const int HEADERY=26;
const int AREAMINY=27;


/**********************************************************
Screen Class:
A UI_screen is a view of the window.  Each UI_screen encompasses
the entire window. An application may have multiple screens.
Each UI_screen is broken down into one or more areas, but there
is at least one. Areas can be created and destroyed and that
causes edges to be created and destroyed.

A change in the window size is the only way to change the
UI_screen size.
**********************************************************/
class UI_screen : public UI_element
{
	public:
		UI_screen();
		UI_screen(const UI_screen &new_screen);  //copy constructor for adding screens
		~UI_screen();
		void draw();
		void process_event(SDL_Event event);

		void set_name(std::string new_name) {screen_name=new_name;}
		std::string get_name() {return screen_name;}

		//are these two needed?
		void set_ID(int new_ID) {ID=new_ID;}
		int get_ID() {return ID;}

		void set_active_area(UI_area* new_area) {active_area=new_area;}
		UI_area* get_active_area() {return active_area;}
		void set_active_edge(UI_edge* new_edge) {active_edge=new_edge;}
		UI_edge* get_active_edge() {return active_edge;}

		void add_area();
		void add_area(UI_area* new_area);

		void add_edge();
		void add_edge(UI_edge* new_edge);

		void join_areas();
		void interactively_split(); //neat drawing code to show the area splitting

	protected:
		UI_edge* get_edge();  //gets the edge that the mouse is over on this UI_screen NULL if not.

		void split_area();  //actually does the splitting of the area

		std::string screen_name;

		std::list<UI_edge*> edges;
		std::list<UI_area*> areas;

		UI_area* active_area;
		UI_edge* active_edge;

		//are these needed?
		int ID;
		struct Scene *scene;
		short startx, endx, starty, endy;	/* framebuffer coords */
		short sizex, sizey;
		short scenenr, screennr;			/* only for pupmenu */
		short full, rt;
		short mainwin, winakt;
};


#endif

